import unittest
import biomart


class BiomartServerTestCase(unittest.TestCase):
    def setUp(self):
        self.server = biomart.BiomartServer(verbose=True)  # url defaults to biomart.PUBLIC_BIOMART_URL

    def testCanConnect(self):
        self.assertTrue(self.server.is_alive)
    
    def testHasMartservice(self):
        self.assertTrue('/martservice' in self.server.url)
    
    def testCanFetchAndShowDatabases(self):
        self.server.show_databases()
        self.assertTrue(len(self.server.databases.keys()) > 0)

    def testCanFetchAndShowDatasets(self):
        self.server.show_datasets()
        self.assertTrue(len(self.server.datasets.keys()) > 0)


def suite():
    loader = unittest.TestLoader()
    test_suite = unittest.TestSuite()
    test_suite.addTest(loader.loadTestsFromTestCase(BiomartServerTestCase))
    return test_suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



class BiomartAttribute(object):
    def __init__(self, name, display_name, attribute_page, is_default = False):
        self.name = name
        self.display_name = display_name
        self.attribute_page = attribute_page
        self.is_default = is_default

    def __repr__(self):
        return "'%s' (page: %s, default: %s)" % (self.display_name, self.attribute_page, self.is_default)



import unittest
import biomart
from biomart.lib import PUBLIC_BIOMART_URL


class BiomartDatasetTestCase(unittest.TestCase):
    def setUp(self):
        self.dataset = biomart.BiomartDataset(PUBLIC_BIOMART_URL, name="uniprot", verbose=True)

    def testCanConnectToDataset(self):
        self.assertTrue(self.dataset.server.is_alive)

    def testCanFetchFilters(self):
        self.dataset.fetch_filters()
        self.dataset.show_filters()
        self.assertTrue(len(self.dataset.filters.keys()) > 0)

    def testCanFetchAttributes(self):
        self.dataset.fetch_attributes()
        self.dataset.show_attributes()
        self.assertTrue(len(self.dataset.attributes.keys()) > 0)

    def testCanSearchWithMultipleFilters(self):
        response = self.dataset.search({
            'filters': {'accession': ['Q9FMA1', 'Q8LFJ9']}
        })
        rows = [line for line in response.iter_lines()]
        self.assertTrue(len(rows) > 0)
        self.assertFalse('Exception' in rows[0].decode('utf-8'))
    

def suite():
    loader = unittest.TestLoader()
    test_suite = unittest.TestSuite()
    test_suite.addTest(loader.loadTestsFromTestCase(BiomartDatasetTestCase))
    return test_suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



from xml.etree.ElementTree import Element, SubElement, tostring, fromstring
import pprint
import biomart


class BiomartDataset(object):
    def __init__(self, *args, **kwargs):
        # dataset specific attributes
        self.name = kwargs.get('name', None)
        if not self.name:
            raise biomart.BiomartException("[BiomartDataset] 'name' is required")

        self.display_name = kwargs.get('display_name', self.name)
        self.interface = kwargs.get('interface', 'default')
        self.verbose = kwargs.get('verbose', False)

        # get related biomart server
        server = kwargs.get('server', None)
        if not server:
            url = args[0]
            server = biomart.BiomartServer(url = url, **kwargs)
        self.server = server

        # get related biomart database
        self.database = kwargs.get('database', None)

        self._filters = {}
        self._attributes = {}

    def __repr__(self):
        return self.display_name

    @property
    def attributes(self):
        if not self._attributes:
            self.fetch_attributes()
        return self._attributes

    @property
    def filters(self):
        if not self._filters:
            self.fetch_filters()
        return self._filters

    def show_filters(self):
        pprint.pprint(self.filters)

    def show_attributes(self):
        pprint.pprint(self.attributes)

    def fetch_filters(self):
        if self.verbose:
            print("[BiomartDataset:'%s'] Fetching filters" % self.name)

        r = self.server.get_request(type="filters", dataset=self.name)
        for line in r.iter_lines():
            line = line.decode('utf8')
            if line:
                line = line.split("\t")
                self._filters[line[0]] = biomart.BiomartFilter(
                    name = line[0],
                    display_name = line[1],
                    accepted_values = line[2],
                    filter_type = line[5],
                )

    def fetch_attributes(self):
        if self.verbose:
            print("[BiomartDataset:'%s'] Fetching attributes" % self.name)

        # retrieve default attributes from the dataset configuration page
        # only retrieve default attributes from the first page since we
        # cannot use attributes declared under different <AttributePage>s
        r = self.server.get_request(type="configuration", dataset=self.name)
        xml = fromstring(r.text)
        default_attributes = []
        for attribute_page in xml.findall('./AttributePage'):
            for attribute in attribute_page.findall('./*/*/AttributeDescription[@default="true"]'):
                default_attributes.append(attribute.get('internalName'))
            if len(default_attributes) > 0:
                break

        # grab attribute details
        r = self.server.get_request(type="attributes", dataset=self.name)
        for line in r.iter_lines():
            line = line.decode('utf8')
            if line:
                line = line.split("\t")
                self._attributes[line[0]] = biomart.BiomartAttribute(
                    name = line[0],
                    display_name = line[1],
                    attribute_page = line[3],
                    is_default = line[0] in default_attributes
                )

    def search(self, params = {}, header = 0, count = False):
        if not isinstance(params, dict):
            raise biomart.BiomartException("'params' argument must be a dict")

        if self.verbose:
            print("[BiomartDataset:'%s'] Searching using following params:" % self.name)
            pprint.pprint(params)

        # read filters and attributes from params
        filters = params.get('filters', {})
        attributes = params.get('attributes', [])

        # check filters
        for filter_name, filter_value in filters.items():
            dataset_filter = self.filters.get(filter_name, None)

            if not dataset_filter:
                self.show_filters()
                raise biomart.BiomartException("The filter '%s' does not exist." % filter_name)

            if len(dataset_filter.accepted_values) > 0 and filter_value not in dataset_filter.accepted_values:
                error_msg = "The value '%s' for filter '%s' cannot be used." % (filter_value, filter_name)
                error_msg += " Use one of: [%s]" % ", ".join(map(str, dataset_filter.accepted_values))
                raise biomart.BiomartException(error_msg)

        # check attributes unless we're only counting
        if not count:
            # no attributes given, use default attributes
            if not attributes:
                attributes = [a.name for a in self.attributes.values() if a.is_default]

            # if no default attributes have been defined, raise an exception
            if not attributes:
                raise biomart.BiomartException("at least one attribute is required, none given")

            for attribute_name in attributes:
                if attribute_name not in self.attributes.keys():
                    self.show_attributes()
                    raise biomart.BiomartException("The attribute '%s' does not exist." % attribute_name)

            # selected attributes must belong to the same attribute page.
            if len(set([self.attributes[a].attribute_page for a in attributes])) > 1:
                self.show_attributes()
                raise biomart.BiomartException("You must use attributes that belong to the same attribute page.")

        # filters and attributes looks ok, start building the XML query
        root = Element('Query')
        root.attrib.update({
            'virtualSchemaName': 'default',  # TODO: use database virtualSchemaName instead (if any error)
            'formatter': 'TSV',
            'header': str(header),
            'uniqueRows': '1',
            'datasetConfigVersion': '0.6',
            'count': count is True and '1' or ''
        })

        dataset = SubElement(root, "Dataset")
        dataset.attrib.update({
            'name': self.name,
            'interface': self.interface
        })

        # Add filters to the XML query
        for filter_name, filter_value in filters.items():
            dataset_filter = self.filters[filter_name]

            filter_elem = SubElement(dataset, "Filter")
            filter_elem.set('name', filter_name)

            if 'boolean_list' == dataset_filter.filter_type:
                if filter_value is True or filter_value.lower() in ('included', 'only'):
                    filter_elem.set('excluded', '0')
                elif filter_value is False or filter_value.lower() == 'excluded':
                    filter_elem.set('excluded', '1')
            else:
                if isinstance(filter_value, list) or isinstance(filter_value, tuple):
                    filter_value = ",".join(map(str, filter_value))
                filter_elem.set('value', str(filter_value))

        # Add attributes to the XML query, unless we're only counting
        if not count:
            for attribute_name in attributes:
                attribute_elem = SubElement(dataset, "Attribute")
                attribute_elem.set('name', str(attribute_name))

        if self.verbose:
            print("[BiomartDataset] search query:\n%s" % tostring(root))

        return self.server.get_request(query = tostring(root))

    def count(self, params = {}):
        r = self.search(params, count = True)
        return int(r.text.strip())



import os
import unittest
from pprint import pprint

import biomart
from biomart.lib import PUBLIC_BIOMART_URL

class BiomartDatabaseTestCase(unittest.TestCase):
    def setUp(self):
        self.database = biomart.BiomartDatabase(PUBLIC_BIOMART_URL, name = 'unimart', verbose=True)
    
    def testCanConnectToDatabase(self):
        self.assertTrue( self.database.server.is_alive )
    
    def testCanFetchDatasets(self):
        self.database.fetch_datasets()
        self.assertTrue( len(self.database.datasets.keys()) > 0 )
    
    def testCanSelectDataset(self):
        uniprot = self.database.datasets['uniprot']
        self.assertTrue(uniprot.count() > 0)

def suite():
    loader = unittest.TestLoader()
    test_suite = unittest.TestSuite()
    test_suite.addTest(loader.loadTestsFromTestCase(BiomartDatabaseTestCase))
    return test_suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



from .server import BiomartServer
from .database import BiomartDatabase
from .dataset import BiomartDataset
from .filter import BiomartFilter
from .attribute import BiomartAttribute

class BiomartException(Exception):
    pass



class BiomartFilter(object):
    def __init__(self, name, display_name, accepted_values, filter_type):
        self.name = name
        self.display_name = display_name
        self.filter_type = filter_type
        self.accepted_values = accepted_values.replace('[', '').replace(']', '')

        if self.accepted_values:
            self.accepted_values = self.accepted_values.split(",")

        if not self.accepted_values and self.filter_type == 'boolean_list':
            self.accepted_values = [True, False, 'excluded', 'included', 'only']

    def __repr__(self):
        return "'%s' (type: %s, values: [%s])" % (
            self.display_name,
            self.filter_type,
            ", ".join(map(str, self.accepted_values))
        )



import biomart


class BiomartDatabase(object):
    def __init__(self, *args, **kwargs):
        server = kwargs.get('server', None)
        if not server:
            url = args[0]
            server = biomart.BiomartServer(url = url, **kwargs)

        self.server = server

        self.name = kwargs.get('name', None)
        if not self.name:
            raise biomart.BiomartException("[BiomartDatabase] 'name' is required")

        self.display_name = kwargs.get('display_name', self.name)
        self.virtual_schema = kwargs.get('virtual_schema', 'default')
        self.verbose = kwargs.get('verbose', False)

        self._datasets = {}
    
    def __repr__(self):
        return self.display_name
    
    @property
    def datasets(self):
        if not self._datasets:
            self.fetch_datasets()
        return self._datasets
    
    def show_datasets(self):
        import pprint
        pprint.pprint(self.datasets)

    def fetch_datasets(self):
        if self.verbose:
            print("[BiomartDatabase:'%s'] Fetching datasets" % self)

        r = self.server.get_request(type = 'datasets', mart = self.name)
        for line in r.iter_lines():
            line = line.decode('utf-8')
            if line:
                cols = line.split("\t")
                if len(cols) > 7:
                    name = cols[1]
                    self._datasets[name] = biomart.BiomartDataset(
                        server = self.server,
                        database = self,
                        name = name,
                        display_name = cols[2],
                        interface = cols[7],
                        verbose = self.verbose,
                    )



import os
import pprint
from xml.etree.ElementTree import fromstring

import requests

import biomart
from .lib import PUBLIC_BIOMART_URL


class BiomartServer(object):
    def __init__(self, url = PUBLIC_BIOMART_URL, **kwargs):
        self._databases = {}
        self._datasets = {}

        if 'martservice' not in url:
            url += '/martservice'
        if not url.startswith('http://'):
            url = 'http://' + url

        self.url = url
        self.http_proxy = kwargs.get('http_proxy', os.environ.get('http_proxy', None))
        self.https_proxy = kwargs.get('https_proxy', os.environ.get('https_proxy', None))
        self._verbose = kwargs.get('verbose', False)

        self.is_alive = False
        self.assert_alive()

    def get_verbose(self):
        return self._verbose

    def set_verbose(self, value):
        if not isinstance(value, bool):
            raise biomart.BiomartException("verbose must be set to a boolean value")

        setattr(self, '_verbose', value)

        # propagate verbose state to databases and datasets objects
        for database in self._databases.values():
            database.verbose = True

        for dataset in self._datasets.values():
            dataset.verbose = True
    verbose = property(get_verbose, set_verbose)
    
    def assert_alive(self):
        if not self.is_alive:
            self.get_request()
            self.is_alive = True
            if self.verbose:
                print("[BiomartServer:'%s'] is alive." % self.url)
    
    @property
    def databases(self):
        if not self._databases:
            self.fetch_databases()
        return self._databases
    
    @property
    def datasets(self):
        if not self._datasets:
            self.fetch_datasets()
        return self._datasets

    def fetch_databases(self):
        if self.verbose:
            print("[BiomartServer:'%s'] Fetching databases" % self.url)

        r = self.get_request(type = 'registry')
        xml = fromstring(r.text)
        for database in xml.findall('MartURLLocation'):
            name = database.attrib['name']

            self._databases[name] = biomart.BiomartDatabase(
                server = self,
                name = name,
                display_name = database.attrib['displayName'],
                virtual_schema = database.attrib['serverVirtualSchema'],
                verbose = self._verbose
            )
    
    def fetch_datasets(self):
        if self.verbose:
            print("[BiomartServer:'%s'] Fetching datasets" % self.url)

        for database in self.databases.values():
            self._datasets.update(database.datasets)
    
    def show_databases(self):
        pprint.pprint(self.databases)
    
    def show_datasets(self):
        pprint.pprint(self.datasets)
    
    def get_request(self, **params):
        proxies = {
            'http': self.http_proxy,
            'https': self.https_proxy
        }
        if params:
            r = requests.get(self.url, params = params, proxies = proxies, stream = True)
        else:
            r = requests.get(self.url, proxies = proxies)
        r.raise_for_status()

        return r



import test_server
import test_database
import test_dataset

def suite():
    import unittest
    import doctest
    suite = unittest.TestSuite()
    suite.addTests(test_server.suite())
    suite.addTests(test_database.suite())
    suite.addTests(test_dataset.suite())
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



from setuptools import setup

setup(
    name='biomart',
    version='0.8.0',
    url='https://github.com/sebriois/biomart',
    author='Sebastien Briois',
    author_email='sebriois@gmail.com',
    packages=['biomart','biomart.lib','biomart.test'],
    keywords='bioinformatics biomart',
    license='BSD',
    description='Python API that consumes the biomart webservice',
    long_description=open('README.txt').read(),
    install_requires=[ "requests>=2.2" ],
    test_suite = 'biomart.test.suite',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
    ],
)
